/*
	Javascript on this file supports Admin page 
	The script performs following functionality
	1. Create new lecture
	2. Copy lecture from existing lecture 
	3. Delete a lecture 
	4. Toggle lecture online flag
*/

var selectedLectureId = 0,
    hasVideo = 0;

// object to return values
function Result(){
	this.success = null;
	this.value= null;	
}

$(document).ready(LoadOnlineAndVideoAvailability);

/*this method updates images in the grid for online bit and video column*/
function LoadOnlineAndVideoAvailability(){
	$(".video_image").each(function() {
  		var online = $(this).data("online");
		if (online == 0){		
			var src = $(this).attr('src');
			$(this).attr("src",src.replace("green_mark","white_mark"));
		}
	});
	
	$(".takeOnline").each(function(){
  		var online = $(this).data("online");
		if (online == 1)
			$(this).hide();
	});
	
	$(".takeOffline").each(function(){
  		var online = $(this).data("online");
		if (online == 0)
			$(this).hide();
	});
}

function TakeOnline(id){
	toggleMessage(false);

	if(! parseInt(id)){
		toggleMessage(true, "Invalid lectureId", true);
		return;
}

	updateToggle(id, 1, function(result){
		if(result && result.success){
			if(result && result.success){
				toggleMessage(true,'Lecture successfully updated!');
				//delay to show message
				setInterval(function(){
					window.location = "admin.maml";
				}, 1000);			
			}
		}
		else{
			toggleMessage(true, result.value, true);
		}
	});
}

function TakeOffline(id){
	toggleMessage(false);

	if(! parseInt(id)){
		toggleMessage(true, "Invalid lectureId", true);
		return;
    }
    
	updateToggle(id, 0, function(result){
		if(result && result.success){
			toggleMessage(true,'Lecture successfully updated!');
			//delay to show message
			setInterval(function(){
				window.location = "admin.maml";
			}, 1000);			
}
		else{
			toggleMessage(true,result.value, true);
}
	});
}

function ShowCreateLecture() {
	toggleMessage(false);
      $("#CreateLecture").show();
}

function HideCreateLecture() {
	$("#CreateLecture").fadeOut(500); ResetLecture();
}

function CreateLecture(){
	var lectureName = $("#lectureName").val().trim(),
	    courseNumber = $("#courseNumber").val().trim();
		
	//validation : required fields	
	if( !(lectureName.length > 0) ) {
		$("#error_lecturename").show();
		return;		
	}
	
	if ( !(courseNumber.length > 0) ){
		$("#error_coursenumber").show();
		return;		
	}
			
	saveCreatedLecture(lectureName, courseNumber, userId, function(result){
		if(result && result.success){
			var resultString=result.value;
			resultString = resultString.trim();
			window.location = "lectureedit.maml?t_lectureid="+resultString;	
		}
		else{
			toggleMessage(true, result.value, true);
		}
	});
	HideCreateLecture();
}
	

function ShowCopyLecture(id, videoFlag) {
	toggleMessage(false);
	
	if(! parseInt(id)){
		toggleMessage(true, "Invalid lectureId", true);
		return;
	}
	
	selectedLectureId = id;
	hasVideo = videoFlag;
	$("#CopyLecture").show();
}

function HideCopyLecture() {
	$("#CopyLecture").fadeOut(500);  ResetLecture();
}

function CopyLecture(){
		var lectureName  = $("#lectureName_copy").val().trim(),
	    courseNumber     =  $("#courseNumber_copy").val().trim();
		
		if( !(lectureName.length>0) ){
			$("#error_lecturename_copy").show();
		     return;			
		}
		
		if( !(courseNumber.length>0) ){
			$("#error_coursenumber_copy").show();
		     return;			
		}
		
	saveCopiedLecture(lectureName, courseNumber, selectedLectureId, userId, hasVideo, function(result){
		if(result && result.success){
			window.location = "lectureedit.maml?t_lectureid="+result.value;			
		}
		else{
			toggleMessage(true,result.value, true);
		}
		
	}, CopyLectureFiles);
	
	HideCopyLecture();				
	
}


function ShowDeleteConfirmation(id, videoFlag) {
	
	toggleMessage(false);
	if(! parseInt(id)){
		toggleMessage(true, "Invalid lectureId", true);
		return;
	}
	selectedLectureId = id;
	hasVideo = videoFlag;
	$("#confirmDelete").fadeIn(200);
}

function HideDeleteConfirmation() {
    $("#confirmDelete").fadeOut(500);
}

function DeleteLecture(){
	//if lecture has files delete them first
	DeleteLectureFiles(hasVideo, function(result){
		if(result && result.success){
		    toggleMessage(true,'Lecture was successfully deleted!');
			//delay to show message
			setInterval(function(){
				window.location = "admin.maml"; 
			}, 1000);			
		}
		else{
			toggleMessage(true,result.value, true);
		}		
	}, deleteLectureEntry);	
	
	HideDeleteConfirmation();
}

/*reset form for create/copy lecture to default values*/
function ResetLecture() {
	$("#error_lecturename").hide();
	$("#error_coursenumber").hide();
	$("#lectureName").val('');
	$("#courseNumber").val('');	
}

function toggleMessage(flag, msgStr, isError){
	if(flag){
		showMessage(msgStr, isError);
	}
	else{
		hideMessage(flag);
	}

}

/* below functions responsible for data interaction */

function saveCreatedLecture(lectureName, courseNumber, userId, callback){
	var obj  = new Result();
	// add lecture to data base
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/addlecture.maml',
		data: {t_lecturename: lectureName, t_coursenumber:courseNumber, t_lectureid:0, t_userid:userId},
		dataType:'text',
		success : function(data){
			if(parseInt(data.trim())){
				if(data.trim() === "-1"){
					obj.success = false;
					obj.value =  "Error: Lecture name " + lectureName + " already exist for the course.";				    
					if(callback){
						callback(obj);
					}	
				}
				else{
					//success
					obj.success = true;
					obj.value = data;
					if(callback){
						callback(obj);
					}	
				}	
			}
			else{
				// error : query fail to yield result
				obj.success = false;
				obj.value = "Error occurred while adding a new lecture.";
				if(callback){
					callback(obj);
				}	
			}			
		},
		//ajax call error
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			obj.success = false;
			obj.value = "Error occurred while saving new lecture!";
			//obj.value = "Error: " + XMLHttpRequest.statusText + ". Error code: " + XMLHttpRequest.status;
			if(callback){
				callback(obj);
			}			
		}
	});

}

function saveCopiedLecture(lectureName, courseNumber, selectedLectureId, userId, hasVideo, callback1, callback2){	
	var obj  = new Result();
		$.ajax({
				type : 'POST',
				url : 'commands/lecture/copylecture.maml',
				data: {t_lecturename: lectureName, t_coursenumber: courseNumber, t_lectureid: selectedLectureId , t_userid: userId},
				dataType:'text',
				success : function(data){
					
			//if(parseInt(insertedLectureId)){
				if(parseInt(data.trim())){
					
				//var insertedLectureId = (data)?data.trim():data;
				if(data.trim() === "-1"){
				} else {
				var insertedLectureId = (data)?data.trim():data;

				
				//success 
				if(callback2){
						if(hasVideo){
						//copy files if lecture has video
						if(callback1){
							callback2(selectedLectureId, insertedLectureId, callback1);	
						}						
						}
						else{
						//no files to copy
						obj.success = true;
						obj.value = insertedLectureId;
						if(callback1){
							callback1(obj);
						}	
					}	
				}	
}
			}
			else{
				//failure
				obj.success = false;
				obj.value = "Error occurred while creating a record for the copied lecture!";
				if(callback1){
					callback1(obj);
						}						
					 }												
				},
				error : function(XMLHttpRequest, textStatus, errorThrown) {		
			//ajax error
			obj.success = false;
			obj.value = "Error occurred while saving copied lecture!";
			if(callback){
				callback(obj);
			}	
				}
			});
						
}

function CopyLectureFiles(selectedLectureId, insertedLectureId, callback){
	var obj  = new Result();	
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/lecturevideo.maml',
		data: {t_command:"copy", t_prevLectureId:selectedLectureId, t_lectureId:insertedLectureId},
		dataType: 'text',
		success : function(data){
			if ( ! ( ( data ) && ( data.trim() === 'success' ) ) ){	
				// error copying files
				obj.success = false;
				obj.value = "Error: " + data;
				if(callback){
					callback(obj);
				}	
			}
			else{
				//files successfully copied
				obj.success = true;
				obj.value = insertedLectureId;
				if(callback){
					callback(obj);
				}	
			}			
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {			
			//ajax error
			obj.success = false;
			obj.value = "Error occurred while copying lecture files!";
			if(callback){
				callback(obj);
			}	
		}
	});
}


function deleteLectureEntry(callback){
	var obj  = new Result();	
		$.ajax({
			type : 'POST',
			url : 'commands/lecture/deletelecture.maml',
			data: {t_lectureId:selectedLectureId},
			success : function(data){
				//success record deleted successfully
				obj.success = true;				
				if(callback){
					callback(obj);
				}
				
			else{
				//error deleting the record
				obj.success = false;
				obj.value = "Error: Invalid lecture id.";
				if(callback){
					callback(obj);
				}	
		}
	},
	error : function(XMLHttpRequest, textStatus, errorThrown) {
			//ajax error
			obj.success = false;
			obj.value = "Error occurred while deleting lecture from the database!";
			if(callback){
				callback(obj);
	}
		}
	});				
}

function DeleteLectureFiles(hasVideo, callback1, callback2){
	var obj  = new Result();
	//check if lecture has video. If yes remove the files.
   if(hasVideo){
	   //delete video files if the lecture has video
	   	$.ajax({
		type : 'POST',
		url : 'commands/lecture/lecturevideo.maml',
		data: {t_command: "delete" ,t_lectureId:selectedLectureId},
		dataType: 'text',
		success : function(data){		
				if(data.trim() === "success" && callback2 && callback1){
					obj.success = true;
					callback2(callback1);				  
			}			
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
				//ajax error
				obj.success = false;
				obj.value = "Error occurred while deleting files for the lecture!";
				if(callback1){
					callback1(obj);
				}
		}
	});
   }
   else{
		if(callback2 && callback1){
			callback2(callback1);
		}	
   }
}

function  updateToggle(lectureId, onlineFlag, callback){
	var obj  = new Result();
	$.ajax({
		type : 'GET',
		url: 'commands/lecture/setonline.maml',		
		data:{t_lectureid:lectureId, t_online: onlineFlag},
		success : function(data){	
			obj.success = true;
			if(callback){
				callback(obj);
			}			
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			//ajax error
			obj.success = false;
			obj.value = (onlineFlag)? "Error occurred while setting lecture on-line!" : "Error occurred while setting lecture off-line!";
			if(callback){
				callback(obj);
		}
}
	});

}

