/*SlidesById is a dictionary for easy access to the slide data by Id*/
var captionsById = {};

/*SlidesByNum is an array of slides indexed by slide number, for easy access to slide by number*/
var captionsByNum = [];

function Caption(){
    this.Id=0; //keep 0 in order to know it's a new slide.
	this.Title = "Edit caption to add content...";
    this.Number=0;
    this.Time=-1;
}

var lecHasCaptions = function(){
	return !(captionsByNum.length == 0);
}


function Caption(id,title,number,time){
    this.Id=id; 
	this.Title=title;
    this.Number=number;
    this.Time=time;
}


function internalAddCaption(caption){
	captionsById[caption.Id] = caption;
    captionsByNum[caption.Number - 1] = caption;
}

function reindexOrderCaptions(){
    captionsByNum = [];
	if(Object.keys(captionsById).length > 0){
		$.each(captionsById,function(index,value){
			captionsByNum[value.Number -1] = value;			
		});	
	}
}

function removeCaptionById(captionId){
	if(parseInt(captionId)){
		delete captionsById[captionId];
	}
}


function resetCaptionHolders(){
	captionsById ={};
	captionsByNum=[];
}

// object to return values
function ResultCaption(){
	this.success = null;
	this.value= null;	
}


/*Gets slides metadata from the server and populates slidesById and slidesByNum collections*/
function getCaptionMetadata(lectureId, callback){
	var obj = new ResultCaption();
	$.ajax({
		url:'commands/caption/getcaptions.maml',
		data: {t_lectureid:lectureId,t_command:'getcaptions'},
		dataType: 'json',
		success: function(data){        	
			$.each(data, function(index,value){
				internalAddCaption(new Caption(value.Id,value.Title,value.Number,value.Time));
			});
			reindexOrderCaptions();
			if(callback){
				obj.success = true;
				obj.value = captionsByNum;
				callback(obj);
			}
		},
		error: function(qXHR, textStatus, errorThrown){		
			//alert(errorThrown); 
			obj.success = false;
			obj.value = "Error occurred while fetching captions!";
			if(callback){
				callback(obj);
			}			
		}
	});
}

function getCaptionContent(lectureId, captionId, callback){
 	var obj = new ResultCaption();
	$.ajax({
		type:'POST',
		url:'commands/caption/getcontent.maml',
		data: {t_lectureid:lectureId,t_captionid:captionId},
        dataType: 'html',
        success: function(data){			
            if(callback){
				obj.success = true;
				obj.value = data;
                callback(obj);
			}
		},
        error: function(qXHR, textStatus, errorThrown){
            //alert(errorThrown);
			obj.success = false;
			obj.value = "Error occurred while fetching caption content!";
			if(callback){
				callback(obj);
			}	
		}
	});
}

function saveCaption(lectureId, caption, callback){
	var obj = new ResultCaption();
	if(caption && caption.Id == 0){
		$.ajax({
			url:'commands/caption/addcaption.maml',
			type:'POST',
			data: {t_lectureid:lectureId, t_captionsec:caption.Time},  
			dataType: 'JSON',		
			success: function(data){	
				if(callback && data && data.Id && data.Number){
					caption.Id = data.Id;
					caption.Number = data.Number;				
					internalAddCaption(caption);
					
					obj.success = true;
					obj.value = caption;					
					callback(obj);
				}            
			},
			error: function(qXHR, textStatus, errorThrown){
				//alert(errorThrown);
				obj.success = false;
				obj.value = "Error occurred while adding a new caption!";
				if(callback){
					callback(obj);
				}	
			}
		});
		
	}
	else{
		$.ajax({
			url:'commands/caption/updatecaption.maml',
			type:'POST',
			data: {t_lectureid:lectureId, t_captionid:caption.Id, t_captionnum:caption.Number, t_captionsec:caption.Time},
			dataType: 'text',		 
			success: function(data){           	
				if(callback){
					obj.success = true;
					obj.value = caption;
					callback(obj);
				}
				//redrawCaption(caption);
			},
			error: function(qXHR, textStatus, errorThrown){
				//alert(errorThrown);
				obj.success = false;
				obj.value = "Error occurred while updating the caption!";
				if(callback){
					callback(obj);
				}	
			}
		});
		
	} 
}

function saveCaptionContent(lectureId, captionId, content, callback){
	var obj = new ResultCaption();

    $.ajax({
		url:'commands/caption/savecontent.maml',
		type:'POST',
        data: {t_lectureid:lectureId, t_captionid:captionId, t_captionbody: content},        
        success: function(data){
			if(callback){
				obj.success = true;
				callback(obj);
			}},
			error: function(qXHR, textStatus, errorThrown){
				//alert(errorThrown);
				obj.success = false;
				obj.value = "Error occurred while saving the caption content!";
				if(callback){
					callback(obj);
				}
			}
	});
}



function saveCaptionOrder(lectureId, orderStr, withTime, callback){
	//str = (withTime) ? str + order[i].Id + ":" + order[i].Number+ ":" + order[i].Time : str + order[i].Id + ":" + order[i].Number; 
	var obj = new ResultCaption(),
	hasTime = (withTime)?1:0; 
	$.ajax({
		url:'commands/caption/saveOrder.maml',
		type: 'POST',
		data: {t_lectureid:lectureId,t_captonidorder:orderStr,t_withTime:hasTime},
		dataType: 'text',		
		success: function(data){			
			obj.success = true;
			if(callback){
				callback(obj);
			}			
		},
		error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while reordering captions!";
			if(callback){
				callback(obj);
			}
		}
	});
}


/*delete caption from server*/
function deleteCaption(lectureId, caption,callback){
	var obj = new ResultCaption();
    $.ajax({
		type:'POST',
		url:'commands/caption/delete.maml',
		data: {t_lectureid:lectureId,t_captionid:caption.Id},
		dataType: 'json',
		success: function(data){
			if(data != null && data.deletedId){
				if(callback){
					removeCaptionById(data.deletedId);
					obj.success = true;
					obj.value = data.deletedId;
					callback(obj);
				}
			}				
		},
		error: function(qXHR, textStatus, errorThrown){
			//alert(errorThrown);
			obj.success = false;
			obj.value = "Error occurred while deleting the caption!";
			if(callback){
				callback(obj);
			}	
		}
	});
	
}

function saveLectureName(lectureId, updatedName, callback){
	var obj = new ResultCaption();
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/editlecturename.maml',
		data:{t_lectureid:lectureId,t_lecturename:updatedName},
		success : function(data){
			if(callback){
				obj.success = true;
				callback(obj);
			}			
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {		
			//alert(textStatus);
			obj.success = false;
			obj.value = "Error occurred while saving the lecture name!";
			if(callback){
				callback(obj);
			}	
		}
	});
}

function saveCourseNumber(lectureId, updatedCourseNumber, callback){
	var obj = new ResultCaption();
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/editcoursenumber.maml',
		data:{t_lectureid:lectureId, t_coursenumber:updatedCourseNumber},
		success : function(data){
			if(callback){
				obj.success = true;
				callback(obj);	
			}				
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			//alert(textStatus);
			obj.success = false;
			obj.value = "Error occurred while saving the course number!";
			if(callback){
				callback(obj);
			}				
		}
	});
}
function bulkUploadCaptions(lectureId, content, overWrite, callback){
	var obj = new ResultCaption(),
	overWriteFlag = (overWrite)?1:0;
	$.ajax({
        url:'commands/caption/uploadcontent.maml',
        type:'POST',
        data: {t_lectureid:lectureId, t_content:content, t_delimeter:"\n",t_overWrite:overWriteFlag},
        dataType: 'text',		 
        success: function(data){
			
			obj.success = true;		
			if(callback){
				callback(obj);
			}			
		},
        error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while uploading captions!";
			if(callback){
				callback(obj);
			}	
		}
	});
	
}

