/*
	This is javacript for Lecture Edit page, supports mostly UI related manipulations.
*/

var currentSlide;

$(document).ready(load);

function load(){
	if( !(lectureId && parseInt(lectureId))){
        return;
	}	
	//initialize lecture details
	initialize();	
}


function  initialize(){
	hideMessage();
	$("#lectureName_span").text(g_LectureName);
	$("#courseNumber_span").text(g_CourseNumber);
	enableDeleteSlide(false);
	toggleComments(g_CommentsEnabled);
	getSlidesMetadata(function(result){
		if(result.success){
			drawSlidesList(result.value);
		}
		else{
			showMessage(result.value, true);
		}
	});
//	initTinyMCE();
}


/*Draws slide list on the left for slide navigation*/
function drawSlidesList(slides){	//slidesByNum
	if(lecHasSlides()){			   
		$.each(slides, function(index, slide){			
			if(slide){
				var elem = drawSlide(slide);
			}
		});
		/*enable drag and drop*/
		$("#slidesListOrdered").sortable({revert:true,axis:"y",cursor: "move", cursorAt: {top:56,left:56},stop:reOrderSlides});
		$("#slidesListOrdered").disableSelection();
		$("#slidesListOrdered li").disableSelection();
		$("#slidesListOrdered li > *").disableSelection();
		$(".slide").disableSelection();
	}
	else{
		showMessage("No slides available for the lecture!");
		$("#deleteSlide").hide();
	}
	
}


function metaDataEdit(ele){
	hideMessage();
	var container = ele.id.split('_'),
    pre = container[0];
	var value = $("#"+pre+"_span").text();
	$("#"+pre+"_read").hide();
	$("#"+pre+"_write").show();	
	if(pre === "slidePreview"){
		tinyMCE.get(pre+"_input").setContent($("#"+pre+"_span").html());
//		tinyMCE.get(pre+"_input").setContent($("#"+pre+"_span").text());
		
		
	}
	else{
		$("#"+pre+"_input").val(value);
	}	
}

function metaDataSave(ele){

	var container = ele.id.split('_'),
	pre = container[0];
	var value = (pre === "slidePreview") 
	? stripslashes(tinyMCE.get(pre+"_input").getContent())
	: $("#"+pre+"_input").val().trim();
	var prevValue 			= $("#"+pre+"_span").text().trim(),
	isValueNotUpdated 	=  (prevValue === value);
	//check if value exist
	if(value.trim().length > 0){
		
		if($("#"+pre+"_input").css("border-Color") == "rgb(255, 0, 0)"){
			$("#"+pre+"_input").css("border-Color", "") ;
		}
		//save
		switch(pre){
			case "lectureName":
			if(! isValueNotUpdated){
				saveLectureName(lectureId, value, function(result){
					if(result && result.success){
						//success
						metaDataPostSave(pre, value);
					}
					else{
						//error
						showMessage(result.value, true);						
					}
				});
			}
			else{
				metaDataPostSave(pre, value);
			}
			break;
			case "courseNumber":
			if(! isValueNotUpdated){
				saveCourseNumber(lectureId, value, function(result){
					if(result && result.success){
						//success
						metaDataPostSave(pre, value);
					}
					else{
						//error
						showMessage(result.value, true);	
					}
				});
			}
			else{
				metaDataPostSave(pre, value);
			}
			break;
			case "slideTitle":
			if(! isValueNotUpdated){
				currentSlide.Title = value;

				updateSlide(lectureId, currentSlide, function(result){
					if(result && result.success){
						metaDataPostSave(pre, value);
					}
					else{
						showMessage(result.value, true)
					}
				});
			}
			else{
				metaDataPostSave(pre, value);
			}
			break;
			case "lectureNotes":
			if(! isValueNotUpdated){
				saveLectureNotes(lectureId, currentSlide.Id,  value, function(result){
					if(result && result.success){
						metaDataPostSave(pre, value);
					}
					else{
						showMessage(result.value, true);
					}
				});
			}
			else{
				metaDataPostSave(pre, value);
			}
			break;
			case "studentNotes":
			if(! isValueNotUpdated){
				saveStudentNotes(lectureId, currentSlide.Id,  value, function(result){
					if(result && result.success){
						metaDataPostSave(pre, value);
					}
					else{
						showMessage(result.value, true)						
					}					
				});
			}
			else{
				metaDataPostSave(pre, value);
			}
			break;
			case "slidePreview":
			saveSlideContent(lectureId, currentSlide, value, function(result){
				if(result && result.success){
					metaDataPostSave(pre, value);
				}	
				else{
					showMessage(result.value, true)
				}
			});
			default:
			break;			
		}
	}
	else{
		$("#"+pre+"_input").css("border-Color", "#FF0000") ;
	}
}

function metaDataPostSave(prefix, value){
	hideMessage();
	$("#"+prefix+"_read").show();
	if(prefix === "slidePreview"){
		$("#"+prefix+"_span").html(value);
		tinyMCE.get(prefix+"_input").setContent("");
	}
	else{
		$("#"+prefix+"_span").text(value);
	}	
	$("#"+prefix+"_input").val("");
	$("#"+prefix+"_write").hide();
	
	
	switch(prefix){
		case "lectureName":
		g_LectureName = value;
		$("#lectureNameTitle").text(g_LectureName);
		break;
		
		case "courseNumber":
		g_CourseNumber =  value;
		$("#courseNumberTitle").text(g_CourseNumber);
		break;
		
		case "slideTitle":
		redrawSlide(currentSlide);
		break;
		
		default:
		break;			
	}
	
}

function metaDataCancel(ele){
	hideMessage();
	var container = ele.id.split('_'),
    pre = container[0];
	
	$("#"+pre+"_read").show();
	
	if(pre === "slidePreview"){
		/*set editor text explicitly to "". 
		If not set will prompt user for leaving the page*/ 
		tinyMCE.get(pre+"_input").setContent("");	
	}
	else{
		$("#"+pre+"_input").val("");
		$("#"+pre+"_input").css("border-Color", "") ;
	}
	
	$("#"+pre+"_write").hide();		
}

function enableDeleteSlide(flag){
	if(flag){
		//enable delete when slide is selected
		$('#deleteSlide').attr("disabled", false).fadeTo("fast", 1);
		$('#deleteSlide').click(showConfirmDelete);	
	}
	else{
		$('#deleteSlide').attr("disabled", true).fadeTo("fast", .4);
		$('#deleteSlide').unbind('click', showConfirmDelete);				
	}
}

function toggleComments(flag){
	if (flag == 1){
		$('#enabledComments').attr('checked','checked');
	}
	else{
		$('#enabledComments').removeAttr('checked');
	}
}

function commentsEnabledUpdated(){
	var comentFlag = ($('#enabledComments').is(":checked"))?1:0;
	changeEnableComments(lectureId, comentFlag, function(result){
		if(!(result && result.success)){
			showMessage(result.value , true);
		}
		else{
			hideMessage();	
		}
	});	
}

function showConfirmDelete(){
	var isChecked = false;
	$("#slidesListOrdered li").each(function(index, item){
		if($("#"+item.id).css("border-Color") === "rgb(255, 0, 0)"){
			isChecked = true;
		}		
	});
	
	if(isChecked){
		$("#confirmDelete").fadeIn(200);	
	}
}

function cancelSlideDelete(){
	$("#confirmDelete").fadeOut(500); 	
}


function slideSelected(slideId){
	if(!lectureId){
		return;
	}
	//set current slide
	currentSlide = slidesById[slideId];
	if(! currentSlide){
		return;
	}
	
	getSlideContent(lectureId, slideId, function(result){
		if(result && result.success){
			slidePreview(result.value);
			hideMessage();
		}
		else{
			showMessage(result.value, true);
		}
	});
	$('.slide').css({"borderColor":"black"});
    $('#slide' + slideId).css({"borderColor":"red"});
	enableDeleteSlide(true);
	//sets scroll bar at the top 
	window.scrollTo(0,0);
}

function toggleSlideContent(flag){	
	if(flag){
		$("#slideContentDiv").show();	
	}
	else{
		$("#slideContentDiv").hide();	
	}
}

function slidePreview(data){
	if(data){
		toggleSlideContent(true);
		
		//title
		if(!($("#slideTitle_span").is(":visible"))){
			metaDataCancel($("#slideTitle_Cancel")[0]);
		}
		$("#slideTitle_span").text(currentSlide.Title);
		
		//content		
		if(!($("#slidePreview_span").is(":visible"))){
			metaDataCancel($("#slidePreview_Cancel")[0]);
		}
		$("#slidePreview_span").html($(data).find('slidebody').prevObject[0].innerHTML);
		
		//slide number
		$("#slidePreviewNumber").text(currentSlide.Number);
		//lecture note
		// below line of code gets lecture notes
		if(!($("#lectureNotes_span").is(":visible"))){
			metaDataCancel($("#lectureNotes_Cancel")[0]);
		}
		$("#lectureNotes_span").html($(data).find('slidebody').prevObject[1].innerHTML);
		//student note
		// below line of code gets student notes
		
		if(!($("#studentNotes_span").is(":visible"))){
			metaDataCancel($("#studentNotes_Cancel")[0]);
		}
		$("#studentNotes_span").html($(data).find('slidebody').prevObject[2].innerHTML);		
		
	}
	else{
		toggleSlideContent(false);
	}
}

function addSlide(lectureId){
	//Initialize slide with default values
	var s = new Slide();
    s.Id = 0;
    s.Title = "Change Title...";
    s.Number = 0;
	s.Time =0;

	saveCreatedSlide(s, function(result){

		if(result && result.success){
			hideMessage();
			var insertedSlide = result.value;
			var elem = drawSlide(insertedSlide);
			var container = $('#slidesListOrdered');
			container.animate({ scrollTop: elem.offset().top - container.offset().top + container.scrollTop() });		
			
			//highlight added slide
			$('.slide').css({"borderColor":"black"});
			$('#slide' + s.Id).css({"borderColor":"red"});
			
			//set current slide
			currentSlide = insertedSlide;

			redrawSlide(insertedSlide);
			//Show content section
			/*Initialize data with empty XML similar to one 
				returned by 'getSlideContent' to avoid database call, as 
			newly added slide will always be empty.*/
			var data = "<slidebody><p></p></slidebody><lectureNotes></lectureNotes><studentNotes></studentNotes>";
			slidePreview(data);
			if(lecHasSlides()){
				$("#deleteSlide").show();
				hideMessage(false);
			}
			enableDeleteSlide(true);
			//if error exist for no slides then clear it
		}
		else{

			showMessage(result.value, true);
		}
	});	
}

function deleteSlide(lectureId){	
	if(lectureId && currentSlide){
	    hideMessage();
		deleteSlideRecord(currentSlide, function(result){
			if(result && result.success){
				$("#slide" + result.value).remove();
				reOrderSlides();
				toggleSlideContent(false);
				currentSlide = null;
				enableDeleteSlide(false);
				if(! lecHasSlides()){
					showMessage("No slides available for the lecture!");
					$("#deleteSlide").hide();
				}
			}
			else{
				showMessage(result.value, true);	
			}
			
		});
	}
	cancelSlideDelete();
}


function drawSlide(slide){
	return $('<li class="slide" id="slide'+ slide.Id + '" data-id="'+ slide.Id + '" onclick="slideSelected(' + slide.Id + ');"><h4>'+ jQuery.trim(slide.Title).substring(0, 15) +'</h4><span class="number">'+ (slide.Number!=0?slide.Number:"")+ '</span></li>').appendTo($("#slidesListOrdered"));
}

function redrawSlide(slide){
	if(currentSlide && slide.Id == currentSlide.Id){
		$("#slidePreviewNumber").html(slide.Number);
	}
	$("#slide"+slide.Id +">h4").html(jQuery.trim(slide.Title).substring(0, 15));
	$("#slide"+slide.Id +" .number").html(slide.Number);	
}

function reOrderSlides(){
	var i=1;
	var slideOrder = new Array();
	$("#slidesListOrdered .slide").each(function(index,elem){
		var id = parseInt($(elem).attr("data-id"));
		var slide = slidesById[id];
		//resetting slide number once slide is delted
		slide.Number = i;		
		slideOrder.push({Id:id,Number:i });
		i++;
	});	
	//3nd argument is for time
//	alert(slideOrder.length);
	updateSlideArrays(lectureId, slideOrder, false, function(result){
		if(!(result && result.success)){
			showMessage(result.value, true);
		}
		else{
			hideMessage();	
		}
		//redraw slides
		if(lecHasSlides){
			$.each(slidesByNum, function(index, slide){
				redrawSlide(slide);
			});
		}
	}); 
}

function stripslashes(str) {
	return (str + '')
	.replace(/\\(.?)/g, function(s, n1) {
		switch (n1) {
			case '\\':
			return '\\';
			case '0':
			return '\u0000';
			case '':
			return '';
			default:
			return n1;
		}
	});
	}
	
	
	
		