/*
	This file supports all slides managment actions related to data content updates such as:
	reorder slides,
	add new slide,
	delete slide,
	save student/lecture notes,
	get slide body,
	get notes,
	get slides metadata,
	save slide body
	
*/

/*Gets slides metadata from the server and populates slidesById and slidesByNum collections*/

/*SlidesById is a dictionary for easy access to the slide data by Id*/
var slidesById = {};

/*SlidesByNum is an array of slides indexed by slide number, for easy access to slide by number*/
var slidesByNum = [];

// object to return values
function Result(){
	this.success = null;
	this.value= null;	
}

var lecHasSlides = function(){	
	return !(slidesByNum.length == 0);
}

function Slide(){
    this.Id=0; //keep 0 in order to know it's a new slide.
    this.Title="";
    this.Number=0;
    this.Time=-1;
}

// helper functions
function Slide(id,title,number,time){
    this.Id=id; 
    this.Title=title;
    this.Number=number;
    this.Time=time;
}

function internalAddSlide(slide){
	slidesById[slide.Id] = slide;
	slidesByNum[slide.Number - 1] = slide;
}

function reindexOrderSlides(){
    slidesByNum = [];
    $.each(slidesById,function(index,value){
        slidesByNum[value.Number -1] = value;
	});
}

function removeSlideById(slideId){
	if(parseInt(slideId)){
		delete slidesById[slideId];
	}
}

function updateSlideArrays(lectureId, slideOrder, withTime, callback){
	if(slideOrder.length === 0){
		slidesByNum = [];
	}
	else{
		// First save slide order in database
		//once successfully saved update structures holding slides
		saveSlideOrder(lectureId, convertOrderArrayToString(slideOrder), withTime, function(result){
			if(result && result.success){
				var i=1;
				
				if(slideOrder && slideOrder.length>0){
					$.each(slideOrder,function(key,value){
						slidesById[value.Id].Number = value.Number;
						slidesByNum[i] = slidesById[value.Id];
						i++;
					});
				}
			}
			callback(result);			
		});		
	}
}

function convertOrderArrayToString(slideOrder, withTime){
	var str = "";
	if(slideOrder){
		
		var len = slideOrder.length;
		for(var i = 0; i<len; i++){
			str = (withTime)? str + slideOrder[i].Id + ":" + slideOrder[i].Number + ":" + slideOrder[i].Time : str + slideOrder[i].Id + ":" + slideOrder[i].Number; 
			if(i< len-1){
				str = str+",";				
			}			
		}		
	}
	return str;
}




//database interaction

function saveLectureName(id, name, callback){
	var obj = new Result();
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/editlecturename.maml',
		data:{t_lectureid:id,t_lecturename:name},
		success : function(data){
			obj.success = true;
			callback(obj);
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			obj.success = false;
			obj.value = "Error occurred while saving the lecture name !"
			if(callback){
				callback(obj);
			}
		}
	});
	
}

function saveCourseNumber(id, name, callback){
	var obj = new Result();
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/editcoursenumber.maml',
		data:{t_lectureid:id, t_coursenumber:name},
		success : function(data){
			obj.success = true;
			callback(obj);
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			obj.success = false;
			obj.value = "Error occurred while saving the course number!" 
			if(callback){
				callback(obj);
			}
		}
	});
}

function changeEnableComments(id, enablecomments, callback){
	var obj = new Result();
	$.ajax({
		type : 'POST',
		url : 'commands/lecture/setenablecomments.maml',
		data:{t_lectureid:id,t_enablecomments:enablecomments},
		success : function(data){
			obj.success  = true;
			if(callback){
				callback(obj);
			}
		},
		error : function(XMLHttpRequest, textStatus, errorThrown) {
			obj.success = false;
			obj.value = (enablecomments)? "Error occurred while enabling content for the lecture!" :"Error occurred while disabling content for the lecture!";			
			if(callback){
				callback(obj);
			}
		}
	});	
}

function saveLectureNotes(lectureId, slideId, notes, callback){
	var obj = new Result();
	$.ajax({
		url:'commands/slide/savelecturenotes.maml',
		type:'POST',
		data: {t_lectureid:lectureId,t_slideid:slideId,t_notes:notes},
		success: function(data){
			if(callback){
				obj.success = true;
				callback(obj);
			}			
		},
		error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while saving notes for the lecture!" 			
			if(callback){
				callback(obj);
			}
		}
	}); 
}

function saveStudentNotes(lectureId, slideId, notes, callback){
	var obj = new Result();
	$.ajax({
		type:'POST',
		url:'commands/slide/savestudentnotes.maml',		
		data: {t_lectureid:lectureId, t_slideid:slideId, t_notes: notes},
		success: function(data){ 
			if(callback){
				obj.success = true;
				callback(obj);
			}				
		},
		error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while saving student notes for the lecture!" 			
			if(callback){
				callback(obj);
			}
		}
	}); 
}


function getSlidesMetadata(callback){
	var obj = new Result();
	$.ajax({
		url:'commands/slide/getslides.maml',
		data: {t_lectureid:lectureId,t_command:'getSlides'},
		dataType: 'json',
		success: function(data){         	
			$.each(data, function(index,value){
				internalAddSlide(new Slide(value.Id,value.Title,value.Number,value.Time));
			});
			if(callback){				
				obj.success = true;
				obj.value = slidesByNum;
				reindexOrderSlides(callback(obj));
			}
		},
		error: function(qXHR, textStatus, errorThrown){
			//"Error: " + qXHR.statusText + ". Error code: " + qXHR.status;
			obj.success = false;
			obj.value = "Error occurred while fetching slides!" 			
			if(callback){
				callback(obj);
			}
		}
	});
}

function getSlideContent(lectureId, slideId, callback){
	var obj = new Result();
	$.ajax({
		type:'POST',
		url:'commands/slide/getcontent.maml',
		data: {t_lectureid:lectureId,t_slideid:slideId},
        dataType: 'html',
        success: function(data){
            if(data && callback){
				obj.success = true;
				obj.value = data
                callback(obj);
			}
		},
        error: function(qXHR, textStatus, errorThrown){
            obj.success = false;
			obj.value = "Error occurred while fetching slide details!" 			
			if(callback){
				callback(obj);
			}
		}
	});
}

function saveCreatedSlide(slideInstance, callback){
  
	var obj = new Result();
	$.ajax({
		url:'commands/slide/addslide.maml',
		type:'POST',
		data: {t_lectureid:lectureId, t_slidesec:slideInstance.Time, t_slidetitle:slideInstance.Title},
		dataType: 'json',		
		success: function(data){      	
			if(data){

				slideInstance.Id = data.Id;
				slideInstance.Number = data.Number;
           
				// add newly added slide to the array
				internalAddSlide(slideInstance);
				if(callback){
					obj.success = true;
					obj.value = slideInstance;
					callback(obj);
				}					
			}
			
		},
		error: function(qXHR, textStatus, errorThrown){

			obj.success = false;
			obj.value = "Error occurred while creating a new slide!" 		
			if(callback){
				callback(obj);
			}
		}
	});	
}
//alert('Exeption: '+errorThrown+ " textStatus: "+ textStatus+ " qHXR: "+ qXHR);

/**delete slide from server*/
function deleteSlideRecord(slide,callback){
	var obj = new Result();
    $.ajax({
		type:'POST',
		url:'commands/slide/delete.maml',        
		data: {t_lectureid:lectureId,t_slideid:slide.Id},
        dataType: 'json',
        success: function(data){
            if(data != null && data.deletedId){
                if(callback){
					obj.success = true;
					obj.value = data.deletedId;
					removeSlideById(obj.value);
                    callback(obj);
				}
			}
		},
        error: function(qXHR, textStatus, errorThrown){
            obj.success = false;
			obj.value = "Error occurred while deleting the slide!" 			
			if(callback){
				callback(obj);
			}
		}
	});
}

function updateSlide(lectureId, selectedSlide, callback){
	var obj = new Result();
	$.ajax({
        url:'commands/slide/updateslide.maml',
        type:'POST',
        data: {t_lectureid:lectureId, t_slideid:selectedSlide.Id, t_slidenum:selectedSlide.Number, t_slidesec:selectedSlide.Time, t_slidetitle:selectedSlide.Title},
        dataType: 'text',		 
        success: function(data){
            if(callback){ 
				obj.success = true;
				obj.value = selectedSlide.Title;
                callback(obj);
			}            
		},
        error: function(qXHR, textStatus, errorThrown){
            obj.success = false;
			obj.value = "Error occurred while updating slide!" 		
			if(callback){
				callback(obj);
			}
		}
	});	
}

function saveSlideContent(lectureId, slide, value, callback){
	var obj = new Result();
	value=encodeURIComponent(value);
    $.ajax({     
        url:'commands/slide/savecontent.maml',
		type:'POST',		
        data: {t_lectureid:lectureId, t_slideid:slide.Id, t_slidecontent: value},       
        success: function(data){
		
			if(callback){
				obj.success = true;
				callback(obj);
			}
		},
		error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while saving slide content!" 			
			if(callback){
				callback(obj);
			}
		}
	});
}

function saveSlideOrder(lectureId, orderStr, withTime, callback){


	var obj = new Result();
	var hasTime = (withTime)?1:0;
	$.ajax({
        url:'commands/slide/saveOrder.maml',
        type: 'POST',
        data: {t_lectureid:lectureId,t_slideidOrder:orderStr,t_withTime:hasTime},
        success: function(data){
			if(callback){
				obj.success = true;
				callback(obj);
			}
		},
        error: function(qXHR, textStatus, errorThrown){
			obj.success = false;
			obj.value = "Error occurred while re-ordering slides!" 
			if(callback){
				callback(obj);
			}
		}
	});	 	
}
